<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2022 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Core\Admin;

use _JchOptimizeVendor\Composer\CaBundle\CaBundle;
use _JchOptimizeVendor\GuzzleHttp\Client;
use _JchOptimizeVendor\GuzzleHttp\Psr7\Uri;
use _JchOptimizeVendor\GuzzleHttp\Psr7\Utils as GuzzleUtils;
use _JchOptimizeVendor\GuzzleHttp\RequestOptions;
use _JchOptimizeVendor\Joomla\Filesystem\Folder;

use function file;

use JchOptimize\Core\SystemUri;
use JchOptimize\Core\Uri\Utils;
use JchOptimize\Platform\Paths;

\defined('_JCH_EXEC') or exit('Restricted access');
class Helper
{
    /**
     * @return null|array|string|string[]
     *
     * @deprecated
     */
    public static function expandFileNameLegacy($sFile)
    {
        $sSanitizedFile = \str_replace('//', '/', $sFile);
        $aPathParts = \pathinfo($sSanitizedFile);
        $sRelFile = \str_replace(['_', '//'], ['/', '_'], $aPathParts['basename']);

        return \preg_replace('#^'.\preg_quote(\ltrim(SystemUri::basePath(), \DIRECTORY_SEPARATOR)).'#', Paths::rootPath().\DIRECTORY_SEPARATOR, $sRelFile);
    }

    public static function expandFileName($file): string
    {
        $sanitizedFile = \str_replace('//', '/', $file);
        $aPathParts = \pathinfo($sanitizedFile);
        $expandedBasename = \str_replace(['_', '//'], [\DIRECTORY_SEPARATOR, '_'], $aPathParts['basename']);

        return Paths::rootPath().\DIRECTORY_SEPARATOR.\ltrim($expandedBasename, \DIRECTORY_SEPARATOR);
    }

    public static function copyImage($src, $dest): bool
    {
        try {
            $client = new Client([RequestOptions::VERIFY => CaBundle::getBundledCaBundlePath()]);
            $uri = Utils::uriFor($src);
            // If uri not absolute we assume a local file
            if (Uri::isAbsolute($uri)) {
                $response = $client->get($uri);
                $srcStream = $response->getBody();
            } else {
                $srcStream = GuzzleUtils::streamFor(GuzzleUtils::tryFopen($src, 'rb'));
            }
            // Let's ensure parent directory for dest exists
            if (!\file_exists(\dirname($dest))) {
                Folder::create(\dirname($dest));
            }
            GuzzleUtils::copyToStream(GuzzleUtils::streamFor($srcStream), GuzzleUtils::streamFor(GuzzleUtils::tryFopen($dest, 'wb')));
        } catch (\Exception $e) {
            return \false;
        }

        return \true;
    }

    /**
     * @return array|string|string[]
     *
     * @deprecated
     */
    public static function contractFileNameLegacy($fileName)
    {
        return \str_replace([Paths::rootPath().\DIRECTORY_SEPARATOR, '_', \DIRECTORY_SEPARATOR], [\ltrim(SystemUri::basePath(), \DIRECTORY_SEPARATOR), '__', '_'], $fileName);
    }

    /**
     * Returns the 'contracted' path of the file relative to the Uri base as opposed to the web root as in legacy.
     *
     * @return array|string|string[]
     */
    public static function contractFileName($filePath)
    {
        return \str_replace([Paths::rootPath().\DIRECTORY_SEPARATOR, '_', \DIRECTORY_SEPARATOR], ['', '__', '_'], $filePath);
    }

    public static function prepareImageUrl(string $image): array
    {
        // return array('path' => Utility::encrypt($image));
        return ['path' => $image];
    }

    public static function stringToBytes($sValue)
    {
        $sUnit = \strtolower(\substr($sValue, -1, 1));

        return (int) $sValue * \pow(1024, \array_search($sUnit, [1 => 'k', 'm', 'g']));
    }

    public static function markOptimized($file)
    {
        $metafile = self::getMetaFile();
        if (!\is_dir(\dirname($metafile))) {
            Folder::create(\dirname($metafile));
        }
        if (!\in_array($file, self::getOptimizedFiles())) {
            \file_put_contents($metafile, $file.\PHP_EOL, \FILE_APPEND);
        }
    }

    public static function getMetaFile(): string
    {
        return Paths::rootPath().\DIRECTORY_SEPARATOR.'.jch'.\DIRECTORY_SEPARATOR.'jch-api2.txt';
    }

    public static function getOptimizedFiles()
    {
        static $optimizeds = null;
        if (\is_null($optimizeds)) {
            $optimizeds = self::getCurrentOptimizedFiles();
        }

        return $optimizeds;
    }

    public static function unmarkOptimized($file)
    {
        $metafile = self::getMetaFile();
        if (!@\file_exists($metafile)) {
            return;
        }
        $aOptimizedFile = self::getCurrentOptimizedFiles();
        if (($key = \array_search($file, $aOptimizedFile)) !== \false) {
            unset($aOptimizedFile[$key]);
        }
        $sContents = \implode(\PHP_EOL, $aOptimizedFile).\PHP_EOL;
        \file_put_contents($metafile, $sContents);
    }

    public static function proOnlyField(): string
    {
        return '<fieldset style="padding: 5px 5px 0 0; color:darkred"><em>Only available in Pro Version!</em></fieldset>';
    }

    protected static function getCurrentOptimizedFiles()
    {
        $metafile = self::getMetaFile();
        if (!\file_exists($metafile)) {
            return [];
        }
        $optimizeds = \file($metafile, \FILE_IGNORE_NEW_LINES);
        if (\false === $optimizeds) {
            $optimizeds = [];
        }

        return $optimizeds;
    }
}
