<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Plugin;

use _JchOptimizeVendor\Joomla\Filesystem\File;
use _JchOptimizeVendor\Joomla\Filesystem\Folder;
use _JchOptimizeVendor\Joomla\Registry\Registry;
use JchOptimize\ContainerFactory;
use JchOptimize\Core\Admin\Tasks;
use JchOptimize\Model\Cache;

class Installer
{
    /**
     * Fires when plugin is activated and create a dir.php file in plugin root containing
     * absolute path of plugin install.
     */
    public function activate(): void
    {
        $file = \JCH_PLUGIN_DIR.'dir.php';
        $absPath = \ABSPATH;
        $code = <<<PHPCODE
<?php
           
\$DIR = '{$absPath}';
           
PHPCODE;
        File::write($file, $code);
        Tasks::leverageBrowserCaching();
        $this->installMUPlugin();
    }

    public function installMUPlugin()
    {
        if (\JCH_PRO) {
            // Copy the mu-plugins in the correct folder
            $mu_folder = $this->getMUPluginDir();
            if (!\is_dir($mu_folder)) {
                Folder::create($mu_folder);
            }
            File::copy(\JCH_PLUGIN_DIR.'mu-plugins/jch-optimize-mode-switcher.php', $mu_folder.'/jch-optimize-mode-switcher.php');
        }
    }

    public function deactivate(): void
    {
        \delete_option('jch-optimize_settings');

        /** @var Cache $cache */
        $container = ContainerFactory::getContainer();
        $cache = $container->get(Cache::class);
        $cache->cleanCache();
        Tasks::cleanHtaccess();
        $this->deleteMUPlugin();
    }

    public function deleteMUPlugin()
    {
        if (\JCH_PRO) {
            $mu_folder = $this->getMUPluginDir();
            if (\defined('WPMU_PLUGIN_DIR') && \WPMU_PLUGIN_DIR) {
                $mu_folder = \WPMU_PLUGIN_DIR;
            }

            try {
                File::delete($mu_folder.'/jch-optimize-mode-switcher.php');
            } catch (\Exception $e) {
            }
        }
    }

    public function updateSettings()
    {
        $container = ContainerFactory::getContainer();

        /** @var Registry $params */
        $params = $container->get('params');
        // Update new Load WEBP setting
        $loadWebp = $params->get('pro_load_webp_images');
        $nextGenImages = $params->get('pro_next_gen_images');
        if (\is_null($loadWebp) && $nextGenImages) {
            $params->set('pro_load_webp_images', '1');
        }
        // Update Exclude JavaScript settings
        $oldJsSettings = ['excludeJs_peo', 'excludeJsComponents_peo', 'excludeScripts_peo', 'excludeJs', 'excludeJsComponents', 'excludeScripts', 'dontmoveJs', 'dontmoveScripts'];
        $updateJsSettings = \false;
        foreach ($oldJsSettings as $oldJsSetting) {
            $oldJsSettingValue = \json_decode(\json_encode($params->get($oldJsSetting)), \true);
            if ($oldJsSettingValue) {
                if (!isset($oldJsSettingValue[0]['url']) && !isset($oldJsSettingValue[0]['script'])) {
                    $updateJsSettings = \true;
                }

                break;
            }
        }
        if ($updateJsSettings) {
            $dontMoveJs = (array) $params->get('excludeJs');
            $dontMoveScripts = (array) $params->get('dontmoveScripts');
            $params->remove('dontmoveJs');
            $params->remove('dontmoveScripts');
            $excludeJsPeoSettingsMap = ['excludeJs_peo' => ['ieo' => 'excludeJs', 'valueType' => 'url', 'dontmove' => $dontMoveJs], 'excludeJsComponents_peo' => ['ieo' => 'excludeJsComponents', 'valueType' => 'url', 'dontmove' => $dontMoveJs], 'excludeScripts_peo' => ['ieo' => 'excludeScripts', 'valueType' => 'script', 'dontmove' => $dontMoveScripts]];
            foreach ($excludeJsPeoSettingsMap as $excludeJsPeoSettingName => $settingsMap) {
                $excludeJsPeoSetting = (array) $params->get($excludeJsPeoSettingName);
                $params->remove($excludeJsPeoSettingName);
                $newExcludeJs_peo = [];
                $i = 0;
                foreach ($excludeJsPeoSetting as $excludeJsPeoSettingValue) {
                    $newExcludeJs_peo[$i][$settingsMap['valueType']] = $excludeJsPeoSettingValue;
                    foreach ($settingsMap['dontmove'] as $dontMoveValue) {
                        if (\false !== \strpos($excludeJsPeoSettingValue, $dontMoveValue)) {
                            $newExcludeJs_peo[$i]['dontmove'] = 'on';
                        }
                    }
                    ++$i;
                }
                $excludeJsIeoSetting = $params->get($settingsMap['ieo']);
                $params->remove($settingsMap['ieo']);
                foreach ($excludeJsIeoSetting as $excludeJsIeoSettingValue) {
                    ++$i;
                    $newExcludeJs_peo[$i][$settingsMap['valueType']] = $excludeJsIeoSettingValue;
                    $newExcludeJs_peo[$i]['ieo'] = 'on';
                    foreach ($settingsMap['dontmove'] as $dontMoveValue) {
                        if (\false !== \strpos($excludeJsIeoSettingValue, $dontMoveValue)) {
                            $newExcludeJs_peo[$i]['dontmove'] = 'on';
                        }
                    }
                }
                $params->set($excludeJsPeoSettingName, $newExcludeJs_peo);
            }
            \update_option('jch-optimize_settings', $params->toArray());
        }
    }

    public function updateMUPlugins()
    {
        $mu_folder = $this->getMUPluginDir();
        $installedMU = $mu_folder.'/jch-optimize-mode-switcher.php';
        if (\file_exists($installedMU) && \md5_file(\JCH_PLUGIN_DIR.'mu-plugins/jch-optimize-mode-switcher.php') !== \md5_file($installedMU)) {
            $this->installMUPlugin();
        }
    }

    private function getMUPluginDir()
    {
        $mu_folder = \ABSPATH.'wp-content/mu-plugins';
        if (\defined('WPMU_PLUGIN_DIR') && \WPMU_PLUGIN_DIR) {
            $mu_folder = \WPMU_PLUGIN_DIR;
        }

        return $mu_folder;
    }
}
